@@ -11,12 +11,10 @@ Python programmer with substantial convenience.
 This module is used to define "PG-API". It creates a set of ABCs
 that makes up the basic interfaces used to work with a PostgreSQL server.
 """
-import collections
-from abc import abstractproperty, abstractmethod
+import collections.abc
+import abc
 
 from .python.element import Element
-from .python.doc import Doc
-from .python.decorlib import propertydoc
 
 __all__ = [
 	'Message',
@@ -57,26 +55,30 @@ class Message(Element):
 		'CLIENT',
 	)
 
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def source(self) -> str:
 		"""
 		Where the message originated from. Normally, 'SERVER', but sometimes
 		'CLIENT'.
 		"""
 
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def code(self) -> str:
 		"""
 		The SQL state code of the message.
 		"""
 
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def message(self) -> str:
 		"""
 		The primary message string.
 		"""
 
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def details(self) -> dict:
 		"""
 		The additional details given with the message. Common keys *should* be the
@@ -94,7 +96,7 @@ class Message(Element):
 		 * 'internal_query'
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def isconsistent(self, other) -> bool:
 		"""
 		Whether the fields of the `other` Message object is consistent with the
@@ -120,21 +122,22 @@ class Result(Element):
 	_e_label = 'RESULT'
 	_e_factors = ('statement', 'parameters', 'cursor_id')
 
-	@abstractmethod
+	@abc.abstractmethod
 	def close(self) -> None:
 		"""
-		Close the Result handle.
+		Close the Result discarding any supporting resources and causing
+		future read operations to emit empty record sets.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def cursor_id(self) -> str:
 		"""
 		The cursor's identifier.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def sql_column_types(self) -> [str]:
 		"""
 		The type of the columns produced by the cursor.
@@ -144,8 +147,8 @@ class Result(Element):
 			['INTEGER', 'CHARACTER VARYING', 'INTERVAL']
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def pg_column_types(self) -> [int]:
 		"""
 		The type Oids of the columns produced by the cursor.
@@ -155,8 +158,8 @@ class Result(Element):
 			[27, 28]
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def column_names(self) -> [str]:
 		"""
 		The attribute names of the columns produced by the cursor.
@@ -166,8 +169,8 @@ class Result(Element):
 			['column1', 'column2', 'emp_name']
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def column_types(self) -> [str]:
 		"""
 		The Python types of the columns produced by the cursor.
@@ -177,8 +180,8 @@ class Result(Element):
 			[<class 'int'>, <class 'str'>]
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def parameters(self) -> (tuple, None):
 		"""
 		The parameters bound to the cursor. `None`, if unknown and an empty tuple
@@ -190,8 +193,8 @@ class Result(Element):
 		`postgresql.api.Database.cursor_from_id`.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def statement(self) -> ("Statement", None):
 		"""
 		The query object used to create the cursor. `None`, if unknown.
@@ -200,18 +203,12 @@ class Result(Element):
 		`postgresql.api.Database.cursor_from_id`.
 		"""
 
-class Chunks(
-	Result,
-	collections.Iterator,
-	collections.Iterable,
-):
+@collections.abc.Iterator.register
+class Chunks(Result):
 	pass
 
-class Cursor(
-	Result,
-	collections.Iterator,
-	collections.Iterable,
-):
+@collections.abc.Iterator.register
+class Cursor(Result):
 	"""
 	A `Cursor` object is an interface to a sequence of tuples(rows). A result
 	set. Cursors publish a file-like interface for reading tuples from a cursor
@@ -235,7 +232,7 @@ class Cursor(
 		False : 'BACKWARD',
 	}
 
-	@abstractmethod
+	@abc.abstractmethod
 	def clone(self) -> "Cursor":
 		"""
 		Create a new cursor using the same factors as `self`.
@@ -244,8 +241,8 @@ class Cursor(
 	def __iter__(self):
 		return self
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def direction(self) -> bool:
 		"""
 		The default `direction` argument for read().
@@ -256,11 +253,8 @@ class Cursor(
 		Cursor operation option.
 		"""
 
-	@abstractmethod
-	def read(self,
-		quantity : "Number of rows to read" = None,
-		direction : "Direction to fetch in, defaults to `self.direction`" = None,
-	) -> ["Row"]:
+	@abc.abstractmethod
+	def read(self, quantity = None, direction = None) -> ["Row"]:
 		"""
 		Read, fetch, the specified number of rows and return them in a list.
 		If quantity is `None`, all records will be fetched.
@@ -273,14 +267,14 @@ class Cursor(
 		quantity, an empty sequence *must* be returned.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __next__(self) -> "Row":
 		"""
 		Get the next tuple in the cursor.
 		Advances the cursor position by one.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def seek(self, offset, whence = 'ABSOLUTE'):
 		"""
 		Set the cursor's position to the given offset with respect to the
@@ -304,133 +298,13 @@ class Cursor(
 		FROM_END will effectively be ABSOLUTE.
 		"""
 
-class Statement(
-	Element,
-	collections.Callable,
-	collections.Iterable,
-):
+class Execution(metaclass = abc.ABCMeta):
 	"""
-	Instances of `Statement` are returned by the `prepare` method of
-	`Database` instances.
-
-	A Statement is an Iterable as well as Callable.
-
-	The Iterable interface is supported for queries that take no arguments at
-	all. It allows the syntax::
-
-		>>> for x in db.prepare('select * FROM table'):
-		...  pass
+	The abstract class of execution methods.
 	"""
-	_e_label = 'STATEMENT'
-	_e_factors = ('database', 'statement_id', 'string',)
 
-	@propertydoc
-	@abstractproperty
-	def statement_id(self) -> str:
-		"""
-		The statment's identifier.
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def string(self) -> object:
-		"""
-		The SQL string of the prepared statement.
-
-		`None` if not available. This can happen in cases where a statement is
-		prepared on the server and a reference to the statement is sent to the
-		client which subsequently uses the statement via the `Database`'s
-		`statement` constructor.
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def sql_parameter_types(self) -> [str]:
-		"""
-		The type of the parameters required by the statement.
-
-		A sequence of `str` objects stating the SQL type name::
-
-			['INTEGER', 'VARCHAR', 'INTERVAL']
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def sql_column_types(self) -> [str]:
-		"""
-		The type of the columns produced by the statement.
-
-		A sequence of `str` objects stating the SQL type name::
-
-			['INTEGER', 'VARCHAR', 'INTERVAL']
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def pg_parameter_types(self) -> [int]:
-		"""
-		The type Oids of the parameters required by the statement.
-
-		A sequence of `int` objects stating the PostgreSQL type Oid::
-
-			[27, 28]
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def pg_column_types(self) -> [int]:
-		"""
-		The type Oids of the columns produced by the statement.
-
-		A sequence of `int` objects stating the SQL type name::
-
-			[27, 28]
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def column_names(self) -> [str]:
-		"""
-		The attribute names of the columns produced by the statement.
-
-		A sequence of `str` objects stating the column name::
-
-			['column1', 'column2', 'emp_name']
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def column_types(self) -> [type]:
-		"""
-		The Python types of the columns produced by the statement.
-
-		A sequence of type objects::
-
-			[<class 'int'>, <class 'str'>]
-		"""
-
-	@propertydoc
-	@abstractproperty
-	def parameter_types(self) -> [type]:
-		"""
-		The Python types expected of parameters given to the statement.
-
-		A sequence of type objects::
-
-			[<class 'int'>, <class 'str'>]
-		"""
-
-	@abstractmethod
-	def clone(self) -> "Statement":
-		"""
-		Create a new statement object using the same factors as `self`.
-
-		When used for refreshing plans, the new clone should replace references to
-		the original.
-		"""
-
-	@abstractmethod
-	def __call__(self, *parameters : "Positional Parameters") -> ["Row"]:
+	@abc.abstractmethod
+	def __call__(self, *parameters) -> ["Row"]:
 		"""
 		Execute the prepared statement with the given arguments as parameters.
 
@@ -441,8 +315,29 @@ class Statement(
 		[...]
 		"""
 
-	@abstractmethod
-	def chunks(self, *parameters) -> collections.Iterable:
+	@abc.abstractmethod
+	def column(self, *parameters) -> collections.abc.Iterable:
+		"""
+		Return an iterator producing the values of first column of the
+		rows produced by the cursor created from the statement bound with the
+		given parameters.
+
+		Column iterators are never scrollable.
+
+		Supporting cursors will be WITH HOLD when outside of a transaction to
+		allow cross-transaction access.
+
+		`column` is designed for the situations involving large data sets.
+
+		Each iteration returns a single value.
+
+		column expressed in sibling terms::
+
+			return map(operator.itemgetter(0), self.rows(*parameters))
+		"""
+
+	@abc.abstractmethod
+	def chunks(self, *parameters) -> collections.abc.Iterable:
 		"""
 		Return an iterator producing sequences of rows produced by the cursor
 		created from the statement bound with the given parameters.
@@ -456,12 +351,12 @@ class Statement(
 		Each iteration returns sequences of rows *normally* of length(seq) ==
 		chunksize. If chunksize is unspecified, a default, positive integer will
 		be filled in. The rows contained in the sequences are only required to
-		support the basic `collections.Sequence` interfaces; simple and quick
+		support the basic `collections.abc.Sequence` interfaces; simple and quick
 		sequence types should be used.
 		"""
 
-	@abstractmethod
-	def rows(self, *parameters) -> collections.Iterable:
+	@abc.abstractmethod
+	def rows(self, *parameters) -> collections.abc.Iterable:
 		"""
 		Return an iterator producing rows produced by the cursor
 		created from the statement bound with the given parameters.
@@ -478,8 +373,8 @@ class Statement(
 			return itertools.chain.from_iterable(self.chunks(*parameters))
 		"""
 
-	@abstractmethod
-	def column(self, *parameters) -> collections.Iterable:
+	@abc.abstractmethod
+	def column(self, *parameters) -> collections.abc.Iterable:
 		"""
 		Return an iterator producing the values of the first column in
 		the cursor created from the statement bound with the given parameters.
@@ -496,15 +391,15 @@ class Statement(
 			return map(operator.itemgetter(0), self.rows(*parameters))
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def declare(self, *parameters) -> Cursor:
 		"""
 		Return a scrollable cursor with hold using the statement bound with the
 		given parameters.
 		"""
 
-	@abstractmethod
-	def first(self, *parameters) -> "'First' object that is returned by the query":
+	@abc.abstractmethod
+	def first(self, *parameters):
 		"""
 		Execute the prepared statement with the given arguments as parameters.
 		If the statement returns rows with multiple columns, return the first
@@ -522,10 +417,8 @@ class Statement(
 		1
 		"""
 
-	@abstractmethod
-	def load_rows(self,
-		iterable : "A iterable of tuples to execute the statement with"
-	):
+	@abc.abstractmethod
+	def load_rows(self, iterable):
 		"""
 		Given an iterable, `iterable`, feed the produced parameters to the
 		query. This is a bulk-loading interface for parameterized queries.
@@ -541,10 +434,8 @@ class Statement(
 		operation can be optimized.
 		"""
 
-	@abstractmethod
-	def load_chunks(self,
-		iterable : "A iterable of chunks of tuples to execute the statement with"
-	):
+	@abc.abstractmethod
+	def load_chunks(self, iterable):
 		"""
 		Given an iterable, `iterable`, feed the produced parameters of the chunks
 		produced by the iterable to the query. This is a bulk-loading interface
@@ -562,25 +453,147 @@ class Statement(
 		that the operation can be optimized.
 		"""
 
-	@abstractmethod
+@collections.abc.Iterator.register
+@collections.abc.Callable.register
+@Execution.register
+class Statement(Element):
+	"""
+	Instances of `Statement` are returned by the `prepare` method of
+	`Database` instances.
+
+	A Statement is an Iterable as well as Callable.
+
+	The Iterable interface is supported for queries that take no arguments at
+	all. It allows the syntax::
+
+		>>> for x in db.prepare('select * FROM table'):
+		...  pass
+	"""
+	_e_label = 'STATEMENT'
+	_e_factors = ('database', 'statement_id', 'string',)
+
+	@property
+	@abc.abstractmethod
+	def statement_id(self) -> str:
+		"""
+		The statment's identifier.
+		"""
+
+	@property
+	@abc.abstractmethod
+	def string(self) -> object:
+		"""
+		The SQL string of the prepared statement.
+
+		`None` if not available. This can happen in cases where a statement is
+		prepared on the server and a reference to the statement is sent to the
+		client which subsequently uses the statement via the `Database`'s
+		`statement` constructor.
+		"""
+
+	@property
+	@abc.abstractmethod
+	def sql_parameter_types(self) -> [str]:
+		"""
+		The type of the parameters required by the statement.
+
+		A sequence of `str` objects stating the SQL type name::
+
+			['INTEGER', 'VARCHAR', 'INTERVAL']
+		"""
+
+	@property
+	@abc.abstractmethod
+	def sql_column_types(self) -> [str]:
+		"""
+		The type of the columns produced by the statement.
+
+		A sequence of `str` objects stating the SQL type name::
+
+			['INTEGER', 'VARCHAR', 'INTERVAL']
+		"""
+
+	@property
+	@abc.abstractmethod
+	def pg_parameter_types(self) -> [int]:
+		"""
+		The type Oids of the parameters required by the statement.
+
+		A sequence of `int` objects stating the PostgreSQL type Oid::
+
+			[27, 28]
+		"""
+
+	@property
+	@abc.abstractmethod
+	def pg_column_types(self) -> [int]:
+		"""
+		The type Oids of the columns produced by the statement.
+
+		A sequence of `int` objects stating the SQL type name::
+
+			[27, 28]
+		"""
+
+	@property
+	@abc.abstractmethod
+	def column_names(self) -> [str]:
+		"""
+		The attribute names of the columns produced by the statement.
+
+		A sequence of `str` objects stating the column name::
+
+			['column1', 'column2', 'emp_name']
+		"""
+
+	@property
+	@abc.abstractmethod
+	def column_types(self) -> [type]:
+		"""
+		The Python types of the columns produced by the statement.
+
+		A sequence of type objects::
+
+			[<class 'int'>, <class 'str'>]
+		"""
+
+	@property
+	@abc.abstractmethod
+	def parameter_types(self) -> [type]:
+		"""
+		The Python types expected of parameters given to the statement.
+
+		A sequence of type objects::
+
+			[<class 'int'>, <class 'str'>]
+		"""
+
+	@abc.abstractmethod
+	def clone(self) -> "Statement":
+		"""
+		Create a new statement object using the same factors as `self`.
+
+		When used for refreshing plans, the new clone should replace references to
+		the original.
+		"""
+
+	@abc.abstractmethod
 	def close(self) -> None:
 		"""
 		Close the prepared statement releasing resources associated with it.
 		"""
 PreparedStatement = Statement
 
-class StoredProcedure(
-	Element,
-	collections.Callable,
-):
+@collections.abc.Callable.register
+class StoredProcedure(Element):
 	"""
 	A function stored on the database.
 	"""
 	_e_label = 'FUNCTION'
 	_e_factors = ('database',)
 
-	@abstractmethod
-	def __call__(self, *args, **kw) -> (object, Cursor, collections.Iterable):
+	@abc.abstractmethod
+	def __call__(self, *args, **kw) -> (object, Cursor, collections.abc.Iterable):
 		"""
 		Execute the procedure with the given arguments. If keyword arguments are
 		passed they must be mapped to the argument whose name matches the key.
@@ -628,8 +641,8 @@ class Transaction(Element):
 	_e_label = 'XACT'
 	_e_factors = ('database',)
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def mode(self) -> (None, str):
 		"""
 		The mode of the transaction block:
@@ -640,8 +653,8 @@ class Transaction(Element):
 		START TRANSACTION statement.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def isolation(self) -> (None, str):
 		"""
 		The isolation level of the transaction block:
@@ -652,7 +665,7 @@ class Transaction(Element):
 		the START TRANSACTION statement.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def start(self) -> None:
 		"""
 		Start the transaction.
@@ -677,7 +690,7 @@ class Transaction(Element):
 		"""
 	begin = start
 
-	@abstractmethod
+	@abc.abstractmethod
 	def commit(self) -> None:
 		"""
 		Commit the transaction.
@@ -690,7 +703,7 @@ class Transaction(Element):
 		If the transaction has already been committed, do nothing.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def rollback(self) -> None:
 		"""
 		Abort the transaction.
@@ -703,13 +716,13 @@ class Transaction(Element):
 		"""
 	abort = rollback
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __enter__(self):
 		"""
 		Run the `start` method and return self.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __exit__(self, typ, obj, tb):
 		"""
 		If an exception is indicated by the parameters, run the transaction's
@@ -730,10 +743,8 @@ class Transaction(Element):
 		block's exit.
 		"""
 
-class Settings(
-	Element,
-	collections.MutableMapping
-):
+@collections.abc.MutableMapping.register
+class Settings(Element):
 	"""
 	A mapping interface to the session's settings. This provides a direct
 	interface to ``SHOW`` or ``SET`` commands. Identifiers and values need
@@ -742,7 +753,7 @@ class Settings(
 	"""
 	_e_label = 'SETTINGS'
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __getitem__(self, key):
 		"""
 		Return the setting corresponding to the given key. The result should be
@@ -750,14 +761,14 @@ class Settings(
 		exist, raise a KeyError.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __setitem__(self, key, value):
 		"""
 		Set the setting with the given key to the given value. The action should
 		be consistent with the effect of the ``SET`` command.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __call__(self, **kw):
 		"""
 		Create a context manager applying the given settings on __enter__ and
@@ -767,14 +778,14 @@ class Settings(
 		...  ...
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def get(self, key, default = None):
 		"""
 		Get the setting with the corresponding key. If the setting does not
 		exist, return the `default`.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def getset(self, keys):
 		"""
 		Return a dictionary containing the key-value pairs of the requested
@@ -782,25 +793,25 @@ class Settings(
 		with the set of keys that did not exist.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def update(self, mapping):
 		"""
 		For each key-value pair, incur the effect of the `__setitem__` method.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def keys(self):
 		"""
 		Return an iterator to all of the settings' keys.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def values(self):
 		"""
 		Return an iterator to all of the settings' values.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def items(self):
 		"""
 		Return an iterator to all of the setting value pairs.
@@ -813,15 +824,15 @@ class Database(Element):
 	"""
 	_e_label = 'DATABASE'
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def backend_id(self) -> (int, None):
 		"""
 		The backend's process identifier.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def version_info(self) -> tuple:
 		"""
 		A version tuple of the database software similar Python's `sys.version_info`.
@@ -830,8 +841,8 @@ class Database(Element):
 		(8, 1, 3, '', 0)
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def client_address(self) -> (str, None):
 		"""
 		The client address that the server sees. This is obtainable by querying
@@ -840,8 +851,8 @@ class Database(Element):
 		`None` if unavailable.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def client_port(self) -> (int, None):
 		"""
 		The client port that the server sees. This is obtainable by querying
@@ -850,25 +861,22 @@ class Database(Element):
 		`None` if unavailable.
 		"""
 
-	@propertydoc
-	@abstractproperty
-	def xact(self,
-		isolation : "ISOLATION LEVEL to use with the transaction" = None,
-		mode : "Mode of the transaction, READ ONLY or READ WRITE" = None,
-	) -> Transaction:
+	@property
+	@abc.abstractmethod
+	def xact(self, isolation = None, mode = None) -> Transaction:
 		"""
 		Create a `Transaction` object using the given keyword arguments as its
 		configuration.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def settings(self) -> Settings:
 		"""
 		A `Settings` instance bound to the `Database`.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def do(language, source) -> None:
 		"""
 		Execute a DO statement using the given language and source.
@@ -877,14 +885,14 @@ class Database(Element):
 		Likely to be a function of Connection.execute.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def execute(sql) -> None:
 		"""
 		Execute an arbitrary block of SQL. Always returns `None` and raise
 		an exception on error.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def prepare(self, sql : str) -> Statement:
 		"""
 		Create a new `Statement` instance bound to the connection
@@ -896,10 +904,15 @@ class Database(Element):
 		(1,)
 		"""
 
-	@abstractmethod
-	def statement_from_id(self,
-		statement_id : "The statement's identification string.",
-	) -> Statement:
+	@abc.abstractmethod
+	def query(self, sql : str, *args) -> Execution:
+		"""
+		Prepare and execute the statement, `sql`, with the given arguments.
+		Equivalent to ``db.prepare(sql)(*args)``.
+		"""
+
+	@abc.abstractmethod
+	def statement_from_id(self, statement_id) -> Statement:
 		"""
 		Create a `Statement` object that was already prepared on the
 		server. The distinction between this and a regular query is that it
@@ -908,14 +921,12 @@ class Database(Element):
 		statement itself.
 		"""
 
-	@abstractmethod
-	def cursor_from_id(self,
-		cursor_id : "The cursor's identification string."
-	) -> Cursor:
+	@abc.abstractmethod
+	def cursor_from_id(self, cursor_id) -> Cursor:
 		"""
 		Create a `Cursor` object from the given `cursor_id` that was already
 		declared on the server.
-		
+
 		`Cursor` objects created this way must *not* be closed when the object
 		is garbage collected. Rather, the user must explicitly close it for
 		the server resources to be released. This is in contrast to `Cursor`
@@ -923,11 +934,8 @@ class Database(Element):
 		`StoredProcedure`.
 		"""
 
-	@abstractmethod
-	def proc(self,
-		procedure_id : \
-			"The procedure identifier; a valid ``regprocedure`` or Oid."
-	) -> StoredProcedure:
+	@abc.abstractmethod
+	def proc(self, procedure_id) -> StoredProcedure:
 		"""
 		Create a `StoredProcedure` instance using the given identifier.
 
@@ -945,7 +953,7 @@ class Database(Element):
 		[1, 2, 3, 4, 5]
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def reset(self) -> None:
 		"""
 		Reset the connection into it's original state.
@@ -960,7 +968,7 @@ class Database(Element):
 		is being recycled.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def notify(self, *channels, **channel_and_payload) -> int:
 		"""
 		NOTIFY the channels with the given payload.
@@ -978,7 +986,7 @@ class Database(Element):
 		keys are the channels and the values are the payloads. Order is undefined.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def listen(self, *channels) -> None:
 		"""
 		Start listening to the given channels.
@@ -986,7 +994,7 @@ class Database(Element):
 		Equivalent to issuing "LISTEN <x>" for x in channels.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def unlisten(self, *channels) -> None:
 		"""
 		Stop listening to the given channels.
@@ -994,14 +1002,14 @@ class Database(Element):
 		Equivalent to issuing "UNLISTEN <x>" for x in channels.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def listening_channels(self) -> ["channel name", ...]:
 		"""
 		Return an *iterator* to all the channels currently being listened to.
 		"""
 
-	@abstractmethod
-	def iternotifies(self, timeout = None) -> collections.Iterator:
+	@abc.abstractmethod
+	def iternotifies(self, timeout = None) -> collections.abc.Iterator:
 		"""
 		Return an iterator to the notifications received by the connection. The
 		iterator *must* produce triples in the form ``(channel, payload, pid)``.
@@ -1023,8 +1031,8 @@ class TypeIO(Element):
 		return ()
 
 class SocketFactory(object):
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def fatal_exception(self) -> Exception:
 		"""
 		The exception that is raised by sockets that indicate a fatal error.
@@ -1033,30 +1041,30 @@ class SocketFactory(object):
 		indicate if that particular exception is actually fatal.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def timeout_exception(self) -> Exception:
 		"""
 		The exception raised by the socket when an operation could not be
 		completed due to a configured time constraint.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def tryagain_exception(self) -> Exception:
 		"""
 		The exception raised by the socket when an operation was interrupted, but
 		should be tried again.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def tryagain(self, err : Exception) -> bool:
 		"""
 		Whether or not `err` suggests the operation should be tried again.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def fatal_exception_message(self, err : Exception) -> (str, None):
 		"""
 		A function returning a string describing the failure, this string will be
@@ -1066,8 +1074,8 @@ class SocketFactory(object):
 		Returns `None` when `err` is not actually fatal.
 		"""
 
-	@abstractmethod
-	def socket_secure(self, socket : "socket object") -> "secured socket":
+	@abc.abstractmethod
+	def socket_secure(self, socket):
 		"""
 		Return a reference to the secured socket using the given parameters.
 
@@ -1076,11 +1084,11 @@ class SocketFactory(object):
 		security.
 		"""
 
-	@abstractmethod
-	def socket_factory_sequence(self) -> [collections.Callable]:
+	@abc.abstractmethod
+	def socket_factory_sequence(self) -> [collections.abc.Callable]:
 		"""
 		Return a sequence of `SocketCreator`s that `Connection` objects will use to
-		create the socket object. 
+		create the socket object.
 		"""
 
 class Category(Element):
@@ -1093,7 +1101,7 @@ class Category(Element):
 	_e_label = 'CATEGORY'
 	_e_factors = ()
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __call__(self, connection):
 		"""
 		Initialize the given connection in order to conform to the category.
@@ -1116,7 +1124,7 @@ class Connector(Element):
 		return self.driver.connection(self, *args, **kw)
 
 	def __init__(self,
-		user : "required keyword specifying the user name(str)" = None,
+		user : str = None,
 		password : str = None,
 		database : str = None,
 		settings : (dict, [(str,str)]) = None,
@@ -1142,16 +1150,16 @@ class Connection(Database):
 	_e_label = 'CONNECTION'
 	_e_factors = ('connector',)
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def connector(self) -> Connector:
 		"""
-		The `Connector` instance facilitating the `Connection` object's
+		The :py:class:`Connector` instance facilitating the `Connection` object's
 		communication and initialization.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def closed(self) -> bool:
 		"""
 		`True` if the `Connection` is closed, `False` if the `Connection` is
@@ -1161,13 +1169,14 @@ class Connection(Database):
 		True
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def clone(self) -> "Connection":
 		"""
 		Create another connection using the same factors as `self`. The returned
 		object should be open and ready for use.
 		"""
 
+	@abc.abstractmethod
 	def connect(self) -> None:
 		"""
 		Establish the connection to the server and initialize the category.
@@ -1178,7 +1187,7 @@ class Connection(Database):
 		if cat is not None:
 			cat(self)
 
-	@abstractmethod
+	@abc.abstractmethod
 	def close(self) -> None:
 		"""
 		Close the connection.
@@ -1186,13 +1195,13 @@ class Connection(Database):
 		Does nothing if the connection is already closed.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __enter__(self):
 		"""
 		Establish the connection and return self.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __exit__(self, typ, obj, tb):
 		"""
 		Closes the connection and returns `False` when an exception is passed in,
@@ -1208,7 +1217,7 @@ class Driver(Element):
 	_e_label = "DRIVER"
 	_e_factors = ()
 
-	@abstractmethod
+	@abc.abstractmethod
 	def connect(**kw):
 		"""
 		Create a connection using the given parameters for the Connector.
@@ -1217,20 +1226,20 @@ class Driver(Element):
 class Installation(Element):
 	"""
 	Interface to a PostgreSQL installation. Instances would provide various
-	information about an installation of PostgreSQL accessible by the Python 
+	information about an installation of PostgreSQL accessible by the Python
 	"""
 	_e_label = "INSTALLATION"
 	_e_factors = ()
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def version(self):
 		"""
 		A version string consistent with what `SELECT version()` would output.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def version_info(self):
 		"""
 		A tuple specifying the version in a form similar to Python's
@@ -1239,16 +1248,16 @@ class Installation(Element):
 		See `postgresql.versionstring`.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def type(self):
 		"""
 		The "type" of PostgreSQL. Normally, the first component of the string
 		returned by pg_config.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def ssl(self) -> bool:
 		"""
 		Whether the installation supports SSL.
@@ -1262,74 +1271,67 @@ class Cluster(Element):
 	_e_label = 'CLUSTER'
 	_e_factors = ('installation', 'data_directory')
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def installation(self) -> Installation:
 		"""
 		The installation used by the cluster.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def data_directory(self) -> str:
 		"""
 		The path to the data directory of the cluster.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def init(self,
-		initdb : "path to the initdb to use" = None,
-		user : "name of the cluster's superuser" = None,
-		password : "superuser's password" = None,
-		encoding : "the encoding to use for the cluster" = None,
-		locale : "the locale to use for the cluster" = None,
-		collate : "the collation to use for the cluster" = None,
-		ctype : "the ctype to use for the cluster" = None,
-		monetary : "the monetary to use for the cluster" = None,
-		numeric : "the numeric to use for the cluster" = None,
-		time : "the time to use for the cluster" = None,
-		text_search_config : "default text search configuration" = None,
-		xlogdir : "location for the transaction log directory" = None,
+		initdb = None,
+		user = None, password = None,
+		encoding = None, locale = None,
+		collate = None, ctype = None,
+		monetary = None, numeric = None, time = None,
+		text_search_config = None,
+		xlogdir = None,
 	):
 		"""
 		Create the cluster at the `data_directory` associated with the Cluster
 		instance.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def drop(self):
 		"""
 		Kill the server and completely remove the data directory.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def start(self):
 		"""
 		Start the cluster.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def stop(self):
 		"""
 		Signal the server to shutdown.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def kill(self):
 		"""
 		Kill the server.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def restart(self):
 		"""
 		Restart the cluster.
 		"""
 
-	@abstractmethod
-	def wait_until_started(self,
-		timeout : "maximum time to wait" = 10
-	):
+	@abc.abstractmethod
+	def wait_until_started(self, timeout = 10):
 		"""
 		After the start() method is ran, the database may not be ready for use.
 		This method provides a mechanism to block until the cluster is ready for
@@ -1339,10 +1341,8 @@ class Cluster(Element):
 		`postgresql.exceptions.ClusterTimeoutError`.
 		"""
 
-	@abstractmethod
-	def wait_until_stopped(self,
-		timeout : "maximum time to wait" = 10
-	):
+	@abc.abstractmethod
+	def wait_until_stopped(self, timeout = 10):
 		"""
 		After the stop() method is ran, the database may still be running.
 		This method provides a mechanism to block until the cluster is completely
@@ -1352,22 +1352,22 @@ class Cluster(Element):
 		`postgresql.exceptions.ClusterTimeoutError`.
 		"""
 
-	@propertydoc
-	@abstractproperty
+	@property
+	@abc.abstractmethod
 	def settings(self):
 		"""
 		A `Settings` interface to the ``postgresql.conf`` file associated with the
 		cluster.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __enter__(self):
 		"""
 		Start the cluster if it's not already running, and wait for it to be
 		readied.
 		"""
 
-	@abstractmethod
+	@abc.abstractmethod
 	def __exit__(self, exc, val, tb):
 		"""
 		Stop the cluster and wait for it to shutdown *iff* it was started by the
@@ -1377,5 +1377,3 @@ class Cluster(Element):
 __docformat__ = 'reStructuredText'
 if __name__ == '__main__':
 	help(__package__ + '.api')
-##
-# vim: ts=3:sw=3:noet:
