@@ -1,6 +1,5 @@
 ##
-# copyright 2009, James William Pye
-# http://python.projects.postgresql.org
+# .exceptions - Exception hierarchy for PostgreSQL database ERRORs.
 ##
 """
 PostgreSQL exceptions and warnings with associated state codes.
@@ -30,24 +29,29 @@ If that fails, it will return `postgresql.exceptions.Error`
 """
 import sys
 import os
-import traceback
 from functools import partial
 from operator import attrgetter
-from . import api as pg_api
+from .message import Message
 from . import sys as pg_sys
 
 PythonException = Exception
 class Exception(Exception):
-	'Base PostgreSQL exception class'
+	"""
+	Base PostgreSQL exception class.
+	"""
 	pass
 
 class LoadError(Exception):
-	'Failed to load a library'
+	"""
+	Failed to load a library.
+	"""
 
 class Disconnection(Exception):
-	'Exception identifying errors that result in disconnection'
+	"""
+	Exception identifying errors that result in disconnection.
+	"""
 
-class Warning(pg_api.Message):
+class Warning(Message):
 	code = '01000'
 	_e_label = property(attrgetter('__class__.__name__'))
 
@@ -61,10 +65,6 @@ class TypeConversionWarning(DriverWarning):
 	'Report a potential issue with a conversion.'
 	code = '01-TP'
 
-class ClusterWarning(Warning):
-	code = ''
-	source = 'CLUSTER'
-
 class DeprecationWarning(Warning):
 	code = '01P01'
 class DynamicResultSetsReturnedWarning(Warning):
@@ -85,13 +85,17 @@ class NoDataWarning(Warning):
 class NoMoreSetsReturned(NoDataWarning):
 	code = '02001'
 
-class Error(pg_api.Message, Exception):
-	'A PostgreSQL Error'
+class Error(Message, Exception):
+	"""
+	A PostgreSQL Error.
+	"""
 	_e_label = 'ERROR'
 	code = ''
 
 	def __str__(self):
-		'Call .sys.errformat(self)'
+		"""
+		Call .sys.errformat(self).
+		"""
 		return pg_sys.errformat(self)
 
 	@property
@@ -100,7 +104,9 @@ class Error(pg_api.Message, Exception):
 		return None if f is None else f in ('PANIC', 'FATAL')
 
 class DriverError(Error):
-	"Errors originating in the driver's implementation."
+	"""
+	Errors originating in the driver's implementation.
+	"""
 	source = 'CLIENT'
 	code = '--000'
 class AuthenticationMethodError(DriverError, Disconnection):
@@ -115,7 +121,9 @@ class InsecurityError(DriverError, Disconnection):
 	"""
 	code = '--SEC'
 class ConnectTimeoutError(DriverError, Disconnection):
-	'Client was unable to esablish a connection in the given time'
+	"""
+	Client was unable to esablish a connection in the given time.
+	"""
 	code = '--TOE'
 
 class TypeIOError(DriverError):
@@ -127,27 +135,14 @@ class ParameterError(TypeIOError):
 	code = '--PIO'
 class ColumnError(TypeIOError):
 	code = '--CIO'
+class CompositeError(TypeIOError):
+	code = '--cIO'
 
 class OperationError(DriverError):
 	"""
 	An invalid operation on an interface element.
 	"""
 	code = '--OPE'
-##
-# Exceptions pertinent to cluster initialization and management
-##
-class ClusterError(Error):
-	source = 'CLUSTER'
-class ClusterInitializationError(ClusterError):
-	pass
-class InitDBError(ClusterInitializationError):
-	"A non-zero result was returned by the initdb command"
-class ClusterStartupError(ClusterError):
-	pass
-class ClusterNotRunningError(ClusterError):
-	pass
-class ClusterTimeoutError(ClusterError):
-	pass
 
 class TransactionError(Error):
 	pass
@@ -163,7 +158,9 @@ class ConnectionDoesNotExistError(ConnectionError):
 	"""
 	code = '08003'
 class ConnectionFailureError(ConnectionError):
-	'Raised when a connection is dropped'
+	"""
+	Raised when a connection is dropped.
+	"""
 	code = '08006'
 
 class ClientCannotConnectError(ConnectionError):
@@ -183,7 +180,9 @@ class TriggeredActionError(Error):
 	code = '09000'
 
 class FeatureError(Error):
-	"Unsupported feature"
+	"""
+	"Unsupported feature.
+	"""
 	code = '0A000'
 
 class TransactionInitiationError(TransactionError):
@@ -206,7 +205,9 @@ class CaseNotFoundError(Error):
 	code = '20000'
 
 class CardinalityError(Error):
-	"Wrong number of rows returned"
+	"""
+	Wrong number of rows returned.
+	"""
 	code = '21000'
 
 class TriggeredDataChangeViolation(Error):
@@ -216,13 +217,17 @@ class AuthenticationSpecificationError(Error, Disconnection):
 	code = '28000'
 
 class DPDSEError(Error):
-	"Dependent Privilege Descriptors Still Exist"
+	"""
+	Dependent Privilege Descriptors Still Exist.
+	"""
 	code = '2B000'
 class DPDSEObjectError(DPDSEError):
 	code = '2BP01'
 
 class SREError(Error):
-	"SQL Routine Exception"
+	"""
+	SQL Routine Exception.
+	"""
 	code = '2F000'
 class FunctionExecutedNoReturnStatementError(SREError):
 	code = '2F005'
@@ -234,7 +239,9 @@ class ReadingDataProhibitedError(SREError):
 	code = '2F004'
 
 class EREError(Error):
-	"External Routine Exception"
+	"""
+	External Routine Exception.
+	"""
 	code = '38000'
 class ContainingSQLNotPermittedError(EREError):
 	code = '38001'
@@ -246,7 +253,9 @@ class ReadingSQLDataNotPermittedError(EREError):
 	code = '38004'
 
 class ERIEError(Error):
-	"External Routine Invocation Exception"
+	"""
+	External Routine Invocation Exception.
+	"""
 	code = '39000'
 class InvalidSQLState(ERIEError):
 	code = '39001'
@@ -258,7 +267,9 @@ class SRFProtocolError(ERIEError):
 	code = '39P02'
 
 class TRError(TransactionError):
-	"Transaction Rollback"
+	"""
+	Transaction Rollback.
+	"""
 	code = '40000'
 class DeadlockError(TRError):
 	code = '40P01'
@@ -271,7 +282,9 @@ class StatementCompletionUnknownError(TRError):
 
 
 class ITSError(TransactionError):
-	"Invalid Transaction State"
+	"""
+	Invalid Transaction State.
+	"""
 	code = '25000'
 class ActiveTransactionError(ITSError):
 	code = '25001'
@@ -284,24 +297,34 @@ class BadIsolationForBranchError(ITSError):
 class NoActiveTransactionForBranchError(ITSError):
 	code = '25005'
 class ReadOnlyTransactionError(ITSError):
-	"Occurs when an alteration occurs in a read-only transaction."
+	"""
+	Occurs when an alteration occurs in a read-only transaction.
+	"""
 	code = '25006'
 class SchemaAndDataStatementsError(ITSError):
-	"Mixed schema and data statements not allowed."
+	"""
+	Mixed schema and data statements not allowed.
+	"""
 	code = '25007'
 class InconsistentCursorIsolationError(ITSError):
-	"The held cursor requires the same isolation."
+	"""
+	The held cursor requires the same isolation.
+	"""
 	code = '25008'
 
 class NoActiveTransactionError(ITSError):
 	code = '25P01'
 class InFailedTransactionError(ITSError):
-	"Occurs when an action occurs in a failed transaction."
+	"""
+	Occurs when an action occurs in a failed transaction.
+	"""
 	code = '25P02'
 
 
 class SavepointError(TransactionError):
-	"Classification error designating errors that relate to savepoints."
+	"""
+	Classification error designating errors that relate to savepoints.
+	"""
 	code = '3B000'
 class InvalidSavepointSpecificationError(SavepointError):
 	code = '3B001'
@@ -310,7 +333,9 @@ class TransactionTerminationError(TransactionError):
 	code = '2D000'
 
 class IRError(Error):
-	"Insufficient Resource Error"
+	"""
+	Insufficient Resource Error.
+	"""
 	code = '53000'
 class MemoryError(IRError, MemoryError):
 	code = '53200'
@@ -320,7 +345,9 @@ class TooManyConnectionsError(IRError):
 	code = '53300'
 
 class PLEError(OverflowError):
-	"Program Limit Exceeded"
+	"""
+	Program Limit Exceeded
+	"""
 	code = '54000'
 class ComplexityOverflowError(PLEError):
 	code = '54001'
@@ -330,7 +357,9 @@ class ArgumentOverflowError(PLEError):
 	code = '54023'
 
 class ONIPSError(Error):
-	"Object Not In Prerequisite State"
+	"""
+	Object Not In Prerequisite State.
+	"""
 	code = '55000'
 class ObjectInUseError(ONIPSError):
 	code = '55006'
@@ -341,7 +370,9 @@ class UnavailableLockError(ONIPSError):
 
 
 class SEARVError(Error):
-	"Syntax Error or Access Rule Violation"
+	"""
+	Syntax Error or Access Rule Violation.
+	"""
 	code = '42000'
 
 class SEARVNameError(SEARVError):
@@ -464,7 +495,9 @@ class SchemaNameError(NameError):
 	code = '3F000'
 
 class ICVError(Error):
-	"Integrity Contraint Violation"
+	"""
+	Integrity Contraint Violation.
+	"""
 	code = '23000'
 class RestrictError(ICVError):
 	code = '23001'
@@ -558,7 +591,9 @@ class EscapeSequenceError(DataError):
 class EscapeCharacterConflictError(DataError):
 	code = '2200B'
 class EscapeCharacterError(DataError):
-	"Invalid escape character"
+	"""
+	Invalid escape character.
+	"""
 	code = '2200C'
 
 class SubstringError(DataError):
@@ -592,7 +627,9 @@ class IndexCorruptedError(InternalError):
 	code = 'XX002'
 
 class SIOError(Error):
-	"System I/O"
+	"""
+	System I/O.
+	"""
 	code = '58000'
 class UndefinedFileError(SIOError):
 	code = '58P01'
@@ -600,13 +637,17 @@ class DuplicateFileError(SIOError):
 	code = '58P02'
 
 class CFError(Error):
-	"Configuration File Error"
+	"""
+	Configuration File Error.
+	"""
 	code = 'F0000'
 class LockFileExistsError(CFError):
 	code = 'F0001'
 
 class OIError(Error):
-	"Operator Intervention"
+	"""
+	Operator Intervention.
+	"""
 	code = '57000'
 class QueryCanceledError(OIError):
 	code = '57014'
@@ -615,14 +656,20 @@ class AdminShutdownError(OIError, Disconnection):
 class CrashShutdownError(OIError, Disconnection):
 	code = '57P02'
 class ServerNotReadyError(OIError, Disconnection):
-	'Thrown when a connection is established to a server that is still starting up.'
+	"""
+	Thrown when a connection is established to a server that is still starting up.
+	"""
 	code = '57P03'
 
 class PLPGSQLError(Error):
-	"Error raised by a PL/PgSQL procedural function"
+	"""
+	Error raised by a PL/PgSQL procedural function.
+	"""
 	code = 'P0000'
 class PLPGSQLRaiseError(PLPGSQLError):
-	"Error raised by a PL/PgSQL RAISE statement."
+	"""
+	Error raised by a PL/PgSQL RAISE statement.
+	"""
 	code = 'P0001'
 class PLPGSQLNoDataFoundError(PLPGSQLError):
 	code = 'P0002'
@@ -634,9 +681,9 @@ class PLPGSQLTooManyRowsError(PLPGSQLError):
 code_to_error = {}
 code_to_warning = {}
 def map_errors_and_warnings(
-	objs : "A iterable of `Warning`s and `Error`'s",
-	error_container : "apply the code to error association to this object" = code_to_error,
-	warning_container : "apply the code to warning association to this object" = code_to_warning,
+	objs,
+	error_container = code_to_error,
+	warning_container = code_to_warning,
 ):
 	"""
 	Construct the code-to-error and code-to-warning associations.
@@ -674,9 +721,9 @@ def map_errors_and_warnings(
 				container[obj.pg_code] = obj
 
 def code_lookup(
-	default : "The object to return when no code or class is found",
-	container : "where to look for the object associated with the code",
-	code : "the code to find the exception for"
+	default,
+	container,
+	code
 ):
 	obj = container.get(code)
 	if obj is None:
@@ -701,5 +748,3 @@ if __name__ == '__main__':
 				)
 			)
 		)
-##
-# vim: ts=3:sw=3:noet:
