@@ -538,12 +538,16 @@ class test_driver(unittest.TestCase):
 		self.assertEqual(tuple(c.pg_column_types), (pg_types.TEXTOID, pg_types.VARCHAROID))
 		self.assertEqual(tuple(c.column_types), (str,str))
 
-		db.execute("CREATE TYPE public.myudt AS (i int)")
+		# Should be pg_temp or sandbox.
+		schema = db.settings['search_path'].split(',')[0]
+		typpath = '"%s"."myudt"' %(schema,)
+
+		db.execute("CREATE TYPE myudt AS (i int)")
 		myudt_oid = db.prepare("select oid from pg_type WHERE typname='myudt'").first()
-		ps = db.prepare("SELECT $1::text AS my_column1, $2::varchar AS my_column2, $3::public.myudt AS my_column3")
+		ps = db.prepare("SELECT $1::text AS my_column1, $2::varchar AS my_column2, $3::myudt AS my_column3")
 		self.assertEqual(tuple(ps.column_names), ('my_column1','my_column2', 'my_column3'))
-		self.assertEqual(tuple(ps.sql_column_types), ('pg_catalog.text', 'CHARACTER VARYING', '"public"."myudt"'))
-		self.assertEqual(tuple(ps.sql_parameter_types), ('pg_catalog.text', 'CHARACTER VARYING', '"public"."myudt"'))
+		self.assertEqual(tuple(ps.sql_column_types), ('pg_catalog.text', 'CHARACTER VARYING', typpath))
+		self.assertEqual(tuple(ps.sql_parameter_types), ('pg_catalog.text', 'CHARACTER VARYING', typpath))
 		self.assertEqual(tuple(ps.pg_column_types), (
 			pg_types.TEXTOID, pg_types.VARCHAROID, myudt_oid)
 		)
@@ -554,7 +558,7 @@ class test_driver(unittest.TestCase):
 		self.assertEqual(tuple(ps.column_types), (str,str,tuple))
 		c = ps.declare('textdata', 'varchardata', (123,))
 		self.assertEqual(tuple(c.column_names), ('my_column1','my_column2', 'my_column3'))
-		self.assertEqual(tuple(c.sql_column_types), ('pg_catalog.text', 'CHARACTER VARYING', '"public"."myudt"'))
+		self.assertEqual(tuple(c.sql_column_types), ('pg_catalog.text', 'CHARACTER VARYING', typpath))
 		self.assertEqual(tuple(c.pg_column_types), (
 			pg_types.TEXTOID, pg_types.VARCHAROID, myudt_oid
 		))
@@ -820,6 +824,27 @@ class test_driver(unittest.TestCase):
 		with db.xact():
 			self.select()
 
+	@pg_tmp
+	def testTransactionAlias(self):
+		self.assertEqual(db.transaction, db.xact)
+
+		try:
+			with db.transaction():
+				db.execute("CREATE TABLE t (i int);")
+				raise Exception('some failure')
+		except:
+			pass
+		else:
+			self.fail("expected exception was not raised")
+
+		try:
+			db.query("select * from t")
+		except:
+			# No table.
+			pass
+		else:
+			self.fail("transaction abort had no effect")
+
 	def cursor_read(self):
 		ps = db.prepare("SELECT i FROM generate_series(0, (2^8)::int - 1) AS g(i)")
 		c = ps.declare()
